/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.Finder;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.FocusEvent;
import com.google.gwt.event.dom.client.FocusHandler;
import com.google.gwt.event.dom.client.KeyCodes;
import com.google.gwt.event.dom.client.KeyPressEvent;
import com.google.gwt.event.dom.client.KeyPressHandler;
import com.google.gwt.event.logical.shared.CloseEvent;
import com.google.gwt.event.logical.shared.CloseHandler;
import com.google.gwt.i18n.client.DateTimeFormat;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.PopupPanel;
import java.util.Date;
import java.util.List;
import org.ow2.orchestra.common.gwt.utils.client.OrchestraGwtException;
import org.ow2.orchestra.common.gwt.utils.client.ui.popup.Popup;
import org.ow2.orchestra.console.GWTConsole.client.Commons.FinderGrid;
import org.ow2.orchestra.console.GWTConsole.client.Commons.IdLabel;
import org.ow2.orchestra.console.GWTConsole.client.Commons.PresenterIF;
import org.ow2.orchestra.console.GWTConsole.client.ConsoleEventBus;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshHandler;
import org.ow2.orchestra.console.GWTConsole.client.FinderPopup.PFinderPopup;
import org.ow2.orchestra.console.GWTConsole.client.FinderPopup.VFinderPopup;
import org.ow2.orchestra.console.GWTConsole.client.ListInstance.MInstance;
import org.ow2.orchestra.console.GWTConsole.client.ListInstanceActivity.MInstanceActivity;
import org.ow2.orchestra.console.GWTConsole.client.ListInstanceActivity.PListInstanceActivity;
import org.ow2.orchestra.console.GWTConsole.client.ListProcess.MProcess;
import org.ow2.orchestra.console.GWTConsole.client.ListProcessActivity.PListProcessActivity;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServices;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServicesAsync;
import org.ow2.orchestra.console.GWTConsole.client.Tab.PTab.Type;
import org.ow2.orchestra.console.GWTConsole.client.TabLayoutPanel.PTabLayoutPanel;


/**
 * Presenter of the Finder
 *
 * @author Jean-Yves Goubet
 */
public class PFinder extends PresenterIF {

  private VFinderIF vFinderIF = null;

  private final OrchestraManagementServicesAsync orchestramanagement = GWT.create(OrchestraManagementServices.class);

  // indexes of the grids, just modify here to not modify the code
  private static final int SERVER_GRID = 0;
  private static final int PROCESS_GRID = 1;
  private static final int INSTANCE_GRID = 2;
  private static final int ACTIVITY_GRID = 3;

  // number of grids
  private static final int GRID_NUMBERS = 4;

  // it's the current column browsed by the user in the finder
  private int selectedColumn = -1;

  private PopupPanel popup = null;

  private PTabLayoutPanel pTabLayoutPanel = null;

  public PFinder(final PTabLayoutPanel pTabLayoutPanel, final VFinderIF view) {
    //
    this.pTabLayoutPanel = pTabLayoutPanel;
    //
    this.bindingView(view);
    //
    this.addViewRefreshHandlerToBus();
    //
    //setSizeMinToPanel();
    //
    this.refreshCurrentList();
    //
    this.addListeners();
    //
    this.setDebugID();
  }


  /**
   * This method adds handler on event bus
   *
   * @author Jean-Yves Goubet
   */
  private void addViewRefreshHandlerToBus() {
    ConsoleEventBus.getEventBus().addHandler(ViewRefreshEvent.TYPE, new ViewRefreshHandler() {
      public void onRefresh(final ViewRefreshEvent viewRefreshEvent) {
        if (viewRefreshEvent.getElementType() == Type.FINDER) {
          PFinder.this.refreshCurrentList();
          if (!viewRefreshEvent.getFocus()) {
            PFinder.this.vFinderIF.getFocusPanel().setFocus(true);
          }
        }
      }
    });
  }


  /**
   * This method refreshes the view
   *
   * @author Jean-Yves Goubet
   */
  protected void refreshCurrentList() {
    this.fillServerGrid();
  }


  /**
   * This method is used to add handlers to browse the finder's grids
   *
   * @author Jean-Yves Goubet
   */
  private void addFocusPanelHandlers() {

    PFinder.this.vFinderIF.getFocusPanel().addKeyPressHandler(new KeyPressHandler() {
      public void onKeyPress(final KeyPressEvent keyPressed) {
        // don't move cursor if popup is displayed
        if (PFinder.this.popup == null || keyPressed.getCharCode() == ' ') {
          FinderGrid grid = PFinder.this.getGrid(PFinder.this.selectedColumn);
          if (keyPressed.getCharCode() == ' ') {
            // code for the space bar
            // check the column
            if (PFinder.this.popup != null) {
              PFinder.this.hidePopup();
            } else if (PFinder.this.selectedColumn > 0) {
              PFinder.this.createPopup(PFinder.this.selectedColumn);
            }
          } else {
            switch (keyPressed.getNativeEvent().getKeyCode()) {

              case KeyCodes.KEY_UP:
                // firstly, check if we can
                if (grid.getSelectedRow() > 0) {
                  PFinder.this.getGrid(PFinder.this.selectedColumn + 1).clear();
                  grid.getCellFormatter().removeStyleName(grid.getSelectedRow(), 0, "focusedRowStyle");
                  grid.setSelectedRow(grid.getSelectedRow() - 1);
                  grid.getCellFormatter().setStyleName(grid.getSelectedRow(), 0, "focusedRowStyle");
                  // 2 cases (3 later when server grid will be used) :
                  // -user is in process grid, so we have to fill the instances corresponding
                  if (PFinder.this.selectedColumn == 1) {
                    PFinder.this.fillInstanceGrid(((IdLabel) grid.getWidget(grid.getSelectedRow(), 0)).getId());
                  }
                  // -user is in instance grid, so we have to fill the activities corresponding
                  else if (PFinder.this.selectedColumn == 2) {
                    PFinder.this.fillActivityGrid(((IdLabel) grid.getWidget(grid.getSelectedRow(), 0)).getId());
                  }

                }
                break;

              case KeyCodes.KEY_DOWN:
                // firstly, check if we can
                if (grid.getSelectedRow() < grid.getListSize() - 1) {
                  PFinder.this.getGrid(PFinder.this.selectedColumn + 1).clear();
                  grid.getCellFormatter().removeStyleName(grid.getSelectedRow(), 0, "focusedRowStyle");
                  grid.setSelectedRow(grid.getSelectedRow() + 1);
                  grid.getCellFormatter().setStyleName(grid.getSelectedRow(), 0, "focusedRowStyle");
                  // 2 cases (3 later when server grid will be used) :
                  // -user is in process grid, so we have to fill the instances corresponding
                  if (PFinder.this.selectedColumn == 1) {
                    PFinder.this.fillInstanceGrid(((IdLabel) grid.getWidget(grid.getSelectedRow(), 0)).getId());
                  }
                  // -user is in instance grid, so we have to fill the activities corresponding
                  else if (PFinder.this.selectedColumn == 2) {
                    PFinder.this.fillActivityGrid(((IdLabel) grid.getWidget(grid.getSelectedRow(), 0)).getId());
                  }
                }
                break;

              case KeyCodes.KEY_RIGHT:
                // firstly, check if we can (list size is checked to forbid the right move on empty list)
                if (PFinder.this.selectedColumn < 3 && (PFinder.this.getGrid(PFinder.this.selectedColumn + 1).getListSize() != -1)) {
                  grid = PFinder.this.getGrid(++PFinder.this.selectedColumn);
                  grid.setSelectedRow(0);
                  grid.getCellFormatter().setStyleName(0, 0, "focusedRowStyle");
                  // 2 cases :
                  // -user switches from server grid to process grid, so we have to fill the instance grid corresponding
                  if (PFinder.this.selectedColumn == 1) {
                    PFinder.this.fillInstanceGrid(((IdLabel) grid.getWidget(0, 0)).getId());
                  }
                  // -user switches from process grid to instance grid, so we have to fill the activity grid corresponding
                  else if (PFinder.this.selectedColumn == 2) {
                    PFinder.this.fillActivityGrid(((IdLabel) grid.getWidget(0, 0)).getId());
                  }
                }
                break;

              case KeyCodes.KEY_LEFT:
                // firstly, check if we can
                if (PFinder.this.selectedColumn > 0) {
                  grid.getCellFormatter().removeStyleName(grid.getSelectedRow(), 0, "focusedRowStyle");
                  grid.setSelectedRow(-1);
                  // clear the grid 2 columns away
                  grid = PFinder.this.getGrid(PFinder.this.selectedColumn + 1);
                  if (grid != null) {
                    grid.clear();
                    grid.setListSize(-1);
                  }
                  grid = PFinder.this.getGrid(--PFinder.this.selectedColumn);
                  grid.getCellFormatter().setStyleName(grid.getSelectedRow(), 0, "focusedRowStyle");
                }
                break;

              case KeyCodes.KEY_ENTER:
                // create a tab
                if (PFinder.this.selectedColumn == 1) {
                  final IdLabel label = (IdLabel) grid.getWidget(grid.getSelectedRow(), 0);
                  new PListProcessActivity(PFinder.this.pTabLayoutPanel, label.getId(), label.getName());
                }
                if (PFinder.this.selectedColumn == 2) {
                  final IdLabel label = (IdLabel) grid.getWidget(grid.getSelectedRow(), 0);
                  new PListInstanceActivity(PFinder.this.pTabLayoutPanel, label.getId(),
                      label.getName() + " " + label.getId());
                }
                break;

              default:
                break;
            }
          }
        }
      }
    });
  }


  /**
   * This method adds handlers on labels
   *
   * @param label : the label to add handlers
   * @author Jean-Yves Goubet
   */
  private void addClickHandlers(final IdLabel label) {
    label.addClickHandler(new ClickHandler() {

      public void onClick(final ClickEvent clickEvent) {
        final int column = label.getColumn();
        final int row = label.getRow();
        // check if the label clicked is already selected
        if (column == PFinder.this.selectedColumn && row == PFinder.this.getGrid(label.getColumn()).getSelectedRow()) {
          if (column == 1) {
            new PListProcessActivity(PFinder.this.pTabLayoutPanel, label.getId(), label.getName());
          }
          if (column == 2) {
            new PListInstanceActivity(PFinder.this.pTabLayoutPanel, label.getId(),
                label.getName() + " " + label.getId());
          }
        }
        FinderGrid grid = null;
        // clear the next grids and remove grid attributes
        for (int i = label.getColumn() + 1; i < PFinder.GRID_NUMBERS; i++) {
          grid = PFinder.this.getGrid(i);
          if (grid.getSelectedRow() != -1) {
            grid.getCellFormatter().removeStyleName(grid.getSelectedRow(), 0, "focusedRowStyle");
          }
          grid.clear();
          grid.setSelectedRow(-1);
          grid.setListSize(-1);
        }
        grid = PFinder.this.getGrid(label.getColumn());
        // then remove style on selected row
        if (grid.getSelectedRow() != -1) {
          grid.getCellFormatter().removeStyleName(grid.getSelectedRow(), 0, "focusedRowStyle");
        }
        grid.setSelectedRow(label.getRow());
        // apply style on label's cell
        grid.getCellFormatter().setStyleName(grid.getSelectedRow(), 0, "focusedRowStyle");
        // change attributes
        PFinder.this.selectedColumn = label.getColumn();
        // and fill the next grid
        switch (label.getColumn()) {
          case PFinder.SERVER_GRID:
            PFinder.this.fillProcessGrid();
            break;

          case PFinder.PROCESS_GRID:
            PFinder.this.fillInstanceGrid(label.getId());
            break;

          case PFinder.INSTANCE_GRID:
            PFinder.this.fillActivityGrid(label.getId());
            break;

          default:
            break;
        }
      }
    });
  }


  /**
   * Method to display the popup filled by rpc calls
   *
   * @param vFinderPopup : the view to insert in the popup
   * @author Jean-Yves Goubet
   */
  protected void displayPopup(final VFinderPopup vFinderPopup) {
    this.popup.setWidget(vFinderPopup);
    this.popup.setGlassEnabled(true);
    this.popup.center();
    this.popup.setAutoHideEnabled(true);
    this.popup.addCloseHandler(new CloseHandler<PopupPanel>() {
      public void onClose(final CloseEvent<PopupPanel> arg0) {
        PFinder.this.hidePopup();
      }
    });
  }


  /**
   * Method to hide the popup
   *
   * @author Jean-Yves Goubet
   */
  protected void hidePopup() {
    this.popup.clear();
    if (this.popup.isVisible()) {
      this.popup.hide();
    }
    this.popup = null;
    PFinder.this.vFinderIF.getFocusPanel().setFocus(true);
  }

  /**
   * Method to create the popup depending on the column selected
   *
   * @param selectedColumn : the column selected in the finder
   * @author Jean-Yves Goubet
   */
  protected void createPopup(final int column) {
    this.popup = new PopupPanel();
    final PFinderPopup pFinderPopup = new PFinderPopup();
    // to handle the close action
    pFinderPopup.getView().getPopupFocusPanel().addFocusHandler(new FocusHandler() {
      public void onFocus(final FocusEvent arg0) {
        PFinder.this.hidePopup();
      }
    });
    switch (column) {

      case PFinder.PROCESS_GRID:
        this.getProcessInfo(pFinderPopup);
        break;

      case PFinder.INSTANCE_GRID:
        this.getInstanceInfo(pFinderPopup);
        break;

      case PFinder.ACTIVITY_GRID:
        this.getActivityInfo(pFinderPopup);
        break;

      default:
        break;
    }
  }


  /**
   * Method to make RPC call to fill the popup view with activity details
   *
   * @author Jean-Yves Goubet
   */
  private void getActivityInfo(final PFinderPopup pFinderPopup) {
    final FinderGrid grid = this.getGrid(PFinder.ACTIVITY_GRID);
    this.orchestramanagement.getActivity(((IdLabel) grid.getWidget(grid.getSelectedRow(), 0)).getId(), new AsyncCallback<MInstanceActivity>() {

      public void onSuccess(final MInstanceActivity instanceActivity) {
        final VFinderPopup vFinderPopup = pFinderPopup.getView();
        vFinderPopup.getLabel1().setText("UUID : " + instanceActivity.getInstanceActivityId());
        vFinderPopup.getLabel2().setText("Name : " + instanceActivity.getInstanceActivityName());
        vFinderPopup.getLabel4().setText("Type : " + instanceActivity.getInstanceActivityType());
        vFinderPopup.getLabel3().setText("State : " + instanceActivity.getInstanceActivityState());
        vFinderPopup.getLabel5().setText("Start Date : " + PFinder.dateFormatter(instanceActivity.getInstanceActivityBeginDate()));
        vFinderPopup.getLabel6().setText("Last Update Date : " + PFinder.dateFormatter(instanceActivity.getInstanceActivityEndDate()));
        PFinder.this.displayPopup(vFinderPopup);
      }

      public void onFailure(final Throwable caught) {
        GWT.log("PFinder - getActivityInfo: onFailure : " + caught.getMessage());
        if (caught instanceof OrchestraGwtException) {
          Popup.show((OrchestraGwtException) caught);
        } else {
          Popup.showError("Unable to access to Orchestra !", caught.getMessage());
        }
      }
    });

  }


  /**
   * Method to make RPC call to fill the popup view with instance details
   *
   * @author Jean-Yves Goubet
   */
  private void getInstanceInfo(final PFinderPopup pFinderPopup) {
    final FinderGrid grid = this.getGrid(PFinder.INSTANCE_GRID);
    this.orchestramanagement.getInstance(((IdLabel) grid.getWidget(grid.getSelectedRow(), 0)).getId(), new AsyncCallback<MInstance>() {

      public void onSuccess(final MInstance instance) {
        final VFinderPopup vFinderPopup = pFinderPopup.getView();
        vFinderPopup.getLabel1().setText("UUID : " + instance.getInstanceId());
        vFinderPopup.getLabel2().setText("Process UUID : " + instance.getProcessId());
        vFinderPopup.getLabel3().setText("Process Name : " + instance.getProcessName());
        vFinderPopup.getLabel4().setText("State : " + instance.getInstanceState());
        vFinderPopup.getLabel5().setText("Start Date : " + PFinder.dateFormatter(instance.getInstanceStartDate()));
        vFinderPopup.getLabel6().setText("Last Update Date : " + PFinder.dateFormatter(instance.getInstanceLastUpdateDate()));
        PFinder.this.displayPopup(vFinderPopup);
      }

      public void onFailure(final Throwable caught) {
        GWT.log("PFinder - getInstanceInfo: onFailure : " + caught.getMessage());
        if (caught instanceof OrchestraGwtException) {
          Popup.show((OrchestraGwtException) caught);
        } else {
          Popup.showError("Unable to access to Orchestra !", caught.getMessage());
        }
      }
    });
  }


  /**
   * Method to make RPC call to fill the popup view with process details
   *
   * @author Jean-Yves Goubet
   */
  private void getProcessInfo(final PFinderPopup pFinderPopup) {
    final FinderGrid grid = this.getGrid(PFinder.PROCESS_GRID);
    this.orchestramanagement.getProcess(((IdLabel) grid.getWidget(grid.getSelectedRow(), 0)).getId(), new AsyncCallback<MProcess>() {

      public void onSuccess(final MProcess process) {
        final VFinderPopup vFinderPopup = pFinderPopup.getView();
        vFinderPopup.getLabel1().setText("UUID : " + process.getProcessId());
        vFinderPopup.getLabel2().setText("Name : " + process.getProcessName());
        vFinderPopup.getLabel3().setText("Namespace : " + process.getProcessNamespace());
        vFinderPopup.getLabel4().setText("Deployment Date : " + PFinder.dateFormatter(process.getProcessDeploymentDate()));
        vFinderPopup.getLabel5().setText("Undeployment Date : " + PFinder.dateFormatter(process.getProcessUndeploymentDate()));
        vFinderPopup.getLabel6().setText("State : " + process.getProcessState());
        PFinder.this.displayPopup(vFinderPopup);
      }

      public void onFailure(final Throwable caught) {
        GWT.log("PFinder - getProcessInfo: onFailure : " + caught.getMessage());
        if (caught instanceof OrchestraGwtException) {
          Popup.show((OrchestraGwtException) caught);
        } else {
          Popup.showError("Unable to access to Orchestra !", caught.getMessage());
        }
      }

    });
  }


  /**
   * This method is used to format a java.util.Date in string to be displayed.
   *
   * @param dateInDate : date in java.util.Date format
   * @return dateInString : date in our own string format
   * @author Jean-Yves Goubet
   */
  public static String dateFormatter(final Date dateInDate) {
    String dateInString = null;
    if (dateInDate == null) {
      dateInString = "N/A";
    } else {
      final DateTimeFormat processDateFormat = DateTimeFormat.getFormat("MM/dd/yyyy 'at' HH:mm:ss");
      dateInString = processDateFormat.format(dateInDate);
    }
    return dateInString;
  }


  /**
   * This method is used to fill the server grid, for now it just fills one row
   *
   * @author Jean-Yves Goubet
   */
  private void fillServerGrid() {
    final FinderGrid grid = this.getGrid(PFinder.SERVER_GRID);
    grid.resize(1, 1);
    final IdLabel label = new IdLabel("Orchestra", null, null, 0, PFinder.SERVER_GRID);
    this.addClickHandlers(label);
    grid.setWidget(0, 0, label);
    grid.getCellFormatter().addStyleName(0, 0, "focusedRowStyle");
    grid.setSelectedRow(0);
    this.fillProcessGrid();
    grid.setListSize(1);
    grid.setSelectedRow(0);
    if (this.selectedColumn == -1) {
      this.selectedColumn = 0;
    }
    // debug id
    this.setServersDebugID();
  }


  /**
   * This method is used to fill the process grid
   *
   * @author Jean-Yves Goubet
   */
  private void fillProcessGrid() {
    this.orchestramanagement.getProcesses(new AsyncCallback<List<MProcess>>() {

      public void onSuccess(final List<MProcess> processList) {
        // to get the process grid
        final FinderGrid grid = PFinder.this.getGrid(PFinder.PROCESS_GRID);
        final int size = processList.size();
        MProcess process;
        if (size > 0) {
          // fix the grid size
          grid.resize(size, 1);
          // and for each process
          for (int i = 0; i < size; i++) {
            process = processList.get(i);
            // fill the grid with process name and id
            final IdLabel label = new IdLabel(process.getProcessName() + "(" + process.getProcessId() + ")",
                process.getProcessId(), process.getProcessName(), i, PFinder.PROCESS_GRID);
            // we check if the process is deployed or not to apply another style
            if (process.getProcessState().equals("undeployed")) {
              label.setStyleName("italicStyle");
            }
            PFinder.this.addClickHandlers(label);
            grid.setWidget(i, 0, label);
          }
          grid.setListSize(size);
          // to handle the refresh case when user is browsing the finder
          if (PFinder.this.selectedColumn >= 1) {
            PFinder.this.fillInstanceGrid(((IdLabel) grid.getWidget(grid.getSelectedRow(), 0)).getId());
          }
        } else {
          // in case there's no process
          grid.resize(1, 1);
          // just print a label in the grid
          final Label label = new Label("No process on Orchestra !");
          grid.setWidget(0, 0, label);
          grid.setListSize(-1);
        }

        // debug id
        PFinder.this.setProcessDebugID();
      }

      public void onFailure(final Throwable caught) {
        GWT.log("PFinder - fillProcessGrid: onFailure : " + caught.getMessage());
        if (caught instanceof OrchestraGwtException) {
          Popup.show((OrchestraGwtException) caught);
        } else {
          Popup.showError("Unable to access to Orchestra !", caught.getMessage());
        }
      }
    });
  }


  /**
   * This method is used to fill the instance grid
   *
   * @param processId : the process UUID to get the corresponding instances
   * @author Jean-Yves Goubet
   */
  private void fillInstanceGrid(final String processId) {
    this.orchestramanagement.getInstances(processId, new AsyncCallback<List<MInstance>>() {

      public void onSuccess(final List<MInstance> instanceList) {
        if (PFinder.this.selectedColumn >= PFinder.INSTANCE_GRID - 1) {
          // to get the instance grid
          final FinderGrid grid = PFinder.this.getGrid(PFinder.INSTANCE_GRID);
          final int size = instanceList.size();
          MInstance instance;
          if (size > 0) {
            // fix the grid size
            grid.resize(size, 1);
            // and for each instance
            for (int i = 0; i < size; i++) {
              // fill the grid with instance id
              instance = instanceList.get(i);

              final IdLabel label = new IdLabel(instance.getInstanceId() + " ["
                  + instance.getInstanceState() + "]", instance.getInstanceId(),
                  instance.getProcessName(), i, PFinder.INSTANCE_GRID);
              // check state to set style to label
              if (!(instance.getInstanceState().equals("running") || instance.getInstanceState().equals("suspended"))) {
                label.setStyleName("italicStyle");
              }
              PFinder.this.addClickHandlers(label);
              grid.setWidget(i, 0, label);
            }
            grid.setListSize(size);
            // to handle the refresh case when user is browsing the finder
            if (PFinder.this.selectedColumn >= 2) {
              PFinder.this.fillActivityGrid(((IdLabel) grid.getWidget(grid.getSelectedRow(), 0)).getId());
            }
          } else {
            // in case there's no instance
            grid.resize(1, 1);
            // just print a label
            final Label label = new Label("No instances for this process !");
            grid.setWidget(0, 0, label);
            grid.setListSize(-1);
          }

          // debug id
          PFinder.this.setInstancesDebugID();
        }
      }

      public void onFailure(final Throwable caught) {
        GWT.log("PFinder - fillInstanceGrid: onFailure : " + caught.getMessage());
        if (caught instanceof OrchestraGwtException) {
          Popup.show((OrchestraGwtException) caught);
        } else {
          Popup.showError("Unable to access to Orchestra !", caught.getMessage());
        }
      }
    });
  }


  /**
   * This method is used to fill the activity grid
   *
   * @param instanceId : The instance UUID to get the corresponding activities
   * @author Jean-Yves Goubet
   */
  private void fillActivityGrid(final String instanceId) {
    this.orchestramanagement.getInstanceActivities(instanceId, new AsyncCallback<List<MInstanceActivity>>() {

      public void onSuccess(final List<MInstanceActivity> instanceActivitiesList) {
        // just to be sure user is always on previous grid
        if (PFinder.this.selectedColumn >= PFinder.ACTIVITY_GRID - 1) {
          // to get the activity grid
          final FinderGrid grid = PFinder.this.getGrid(PFinder.ACTIVITY_GRID);
          final int size = instanceActivitiesList.size();
          MInstanceActivity instanceActivity;
          if (size > 0) {
            // fix the grid size
            grid.resize(size, 1);
            // and for each activity
            for (int i = 0; i < size; i++) {
              instanceActivity = instanceActivitiesList.get(i);
              // fill in the grid with activity name
              final IdLabel label = new IdLabel(instanceActivity.getInstanceActivityName(),
                  instanceActivity.getInstanceActivityId(), null, i, PFinder.ACTIVITY_GRID);
              PFinder.this.addClickHandlers(label);
              grid.setWidget(i, 0, label);
            }
            grid.setListSize(size);
          } else {
            // in case there's no activities
            grid.resize(1, 1);
            // just print a label
            final Label label = new Label("No activities for this instance !");
            grid.setWidget(0, 0, label);
            grid.setListSize(-1);
          }

          // debug id
          PFinder.this.setActivitiesDebugID();
        }
      }

      public void onFailure(final Throwable caught) {
        GWT.log("PFinder - fillActivityGrid: onFailure : " + caught.getMessage());
        if (caught instanceof OrchestraGwtException) {
          Popup.show((OrchestraGwtException) caught);
        } else {
          Popup.showError("Unable to access to Orchestra !", caught.getMessage());
        }
      }
    });
  }


  /**
   * This method is used to get the appropriate grid
   *
   * @param whichGrid : the number of the grid (defined in class variables)
   * @return the FinderGrid corresponding
   * @author Jean-Yves Goubet
   */
  private FinderGrid getGrid(final int whichGrid) {
    switch (whichGrid) {

      case 0:
        return this.vFinderIF.getServerGrid();

      case 1:
        return this.vFinderIF.getProcessGrid();

      case 2:
        return this.vFinderIF.getInstanceGrid();

      case 3:
        return this.vFinderIF.getActivityGrid();

      default:
        return null;
    }
  }

/*  unused because of center child of SplitLayoutPanel problem
  /**
   * This method is used to set a width min for all child widget of split layout panel. Doesn't work for center child, that's why
   * center is for server grid which is not implemented yet.
   *
   * @author Jean-Yves Goubet
   *
   *
  private void setSizeMinToPanel() {
    SplitLayoutPanel splitLayoutPanel = this.vFinderIF.getSplitLayoutPanel();
    for (int i = 1;i < PFinder.GRID_NUMBERS;i++) {
      // columns min size set to 50 px except first one
      splitLayoutPanel.setWidgetMinSize(getGrid(i), 50);
    }
  }
*/


  /**
   * Method used to bind the view to the presenter
   *
   * @author Jean-Yves Goubet
   */
  @Override
  protected void bindingView(final Object view) {
    this.vFinderIF = (VFinderIF) view;
  }


  /**
   * Method used to add handlers
   *
   * @author Jean-Yves Goubet
   */
  @Override
  protected void addListeners() {
    this.addFocusPanelHandlers();
  }


  @Override
  protected void setDebugID() {
    this.vFinderIF.getFocusPanel().ensureDebugId("FD-Focus");
  }

  @Override
  public void destroy() {
  }


  /**
   * Method to give id to elements
   *
   * @author Mathieu Crassin
   */
  private void setServersDebugID() {
    final FinderGrid serversGrid = this.getGrid(0);
    if (serversGrid != null) {
      final int imax = serversGrid.getRowCount();
      if (imax > 0) {
        for (int i = 0; i < imax; i++) {
          serversGrid.getWidget(i, 0).ensureDebugId("FD-ServerN" + i);
        }
      }
    }
  }


  /**
   * Method to give id to elements
   *
   * @author Mathieu Crassin
   */
  private void setProcessDebugID() {
    final FinderGrid processGrid = this.getGrid(1);
    if (processGrid != null) {
      final int imax = processGrid.getRowCount();
      if (imax > 0) {
        for (int i = 0; i < imax; i++) {
          processGrid.getWidget(i, 0).ensureDebugId("FD-ProcessN" + i);
        }
      }
    }
  }


  /**
   * Method to give id to elements
   *
   * @author Mathieu Crassin
   */
  private void setInstancesDebugID() {
    final FinderGrid instancesGrid = this.getGrid(2);
    if (instancesGrid != null) {
      final int imax = instancesGrid.getRowCount();
      if (imax > 0) {
        for (int i = 0; i < imax; i++) {
          instancesGrid.getWidget(i, 0).ensureDebugId("FD-InstanceN" + i);
        }
      }
    }
  }


  /**
   * Method to give id to elements
   *
   * @author Mathieu Crassin
   */
  private void setActivitiesDebugID() {
    final FinderGrid activityGrid = this.getGrid(3);
    if (activityGrid != null) {
      final int imax = activityGrid.getRowCount();
      if (imax > 0) {
        for (int i = 0; i < imax; i++) {
          activityGrid.getWidget(i, 0).ensureDebugId("FD-ActivityN" + i);
        }
      }
    }
  }
}
