/**
 * Copyright (C) 2011  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.console.GWTConsole.client.ListInstanceActivity;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.cellview.client.Column;
import com.google.gwt.user.cellview.client.ColumnSortEvent.ListHandler;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.view.client.ListDataProvider;
import java.util.Date;
import java.util.List;
import org.ow2.orchestra.common.gwt.utils.client.OrchestraGwtException;
import org.ow2.orchestra.common.gwt.utils.client.ui.popup.Popup;
import org.ow2.orchestra.console.GWTConsole.client.Commons.celltable.ConsoleCellTable;
import org.ow2.orchestra.console.GWTConsole.client.Commons.celltable.LabelCell;
import org.ow2.orchestra.console.GWTConsole.client.Commons.celltable.NullableDateCell;
import org.ow2.orchestra.console.GWTConsole.client.ConsoleEventBus;
import org.ow2.orchestra.console.GWTConsole.client.Event.RefreshDoneEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.SuspendTimerEvent;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServices;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServicesAsync;
import org.ow2.orchestra.console.GWTConsole.client.ProcessDetails.PProcessViewerPanel;
import org.ow2.orchestra.console.GWTConsole.client.Tab.PTab;
import org.ow2.orchestra.console.GWTConsole.client.TabLayoutPanel.PTabLayoutPanel;


/**
 * @author Guillaume Porcher
 */
public class PListInstanceActivitiesCellTable {


  protected OrchestraManagementServicesAsync orchestramanagement = GWT.create(OrchestraManagementServices.class);

  // Create a data provider.
  private final ListDataProvider<MInstanceActivity> dataProvider = new ListDataProvider<MInstanceActivity>();

  private final PTabLayoutPanel tabLayoutPanel;

  private final String instanceId;
  private final String debugPrefix;

  private final PProcessViewerPanel pProcessViewerPanel;

  public PListInstanceActivitiesCellTable(
          final String instanceId,
          final PTabLayoutPanel ptabLayoutPanel,
          final ConsoleCellTable<MInstanceActivity> table,
          final String debugPrefix,
          final PProcessViewerPanel processViewHandler) {
    this.instanceId = instanceId;
    this.tabLayoutPanel = ptabLayoutPanel;
    this.debugPrefix = debugPrefix;

    this.pProcessViewerPanel = processViewHandler;

    table.ensureDebugId(debugPrefix);

    // Add a ColumnSortEvent.ListHandler to connect sorting to the
    // java.util.List.
    final ListHandler<MInstanceActivity> columnSortHandler = new ListHandler<MInstanceActivity>(this.dataProvider.getList());
    table.addColumnSortHandler(columnSortHandler);

    this.setColumns(table, columnSortHandler);

    // Connect the table to the data provider.
    this.dataProvider.addDataDisplay(table);

    // Fetch data
    this.refresh();

  }

  private void setColumns(final ConsoleCellTable<MInstanceActivity> table,
                          final ListHandler<MInstanceActivity> columnSortHandler) {

    /* Name column */
    final Column<MInstanceActivity, String> nameColumn = new Column<MInstanceActivity, String>(new LabelCell(this.debugPrefix)) {
      @Override
      public String getValue(final MInstanceActivity activity) {
        return activity.getInstanceActivityName();
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, nameColumn, "Name");

    /* Type column */
    final Column<MInstanceActivity, String> typeColumn = new Column<MInstanceActivity, String>(new LabelCell(this.debugPrefix)) {
      @Override
      public String getValue(final MInstanceActivity activity) {
        return activity.getInstanceActivityType();
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, typeColumn, "Type");

    /* State column */
    final Column<MInstanceActivity, String> stateColumn = new Column<MInstanceActivity, String>(new LabelCell(this.debugPrefix)) {
      @Override
      public String getValue(final MInstanceActivity activity) {
        return activity.getInstanceActivityState();
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, stateColumn, "State");


    /* Start date column */
    final Column<MInstanceActivity, Date> startDateColumn = new Column<MInstanceActivity, Date>(new NullableDateCell(this.debugPrefix)) {
      @Override
      public Date getValue(final MInstanceActivity activity) {
        return activity.getInstanceActivityBeginDate();
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, startDateColumn, "Start Date");

    /* End date column */
    final Column<MInstanceActivity, Date> endDateColumn = new Column<MInstanceActivity, Date>(new NullableDateCell(this.debugPrefix)) {
      @Override
      public Date getValue(final MInstanceActivity activity) {
        return activity.getInstanceActivityEndDate();
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, endDateColumn, "End Date");
  }

  public void refresh() {
    this.orchestramanagement.getInstanceActivities(this.instanceId, new AsyncCallback<List<MInstanceActivity>>() {
      public void onSuccess(final List<MInstanceActivity> mInstanceActivityList) {
        PListInstanceActivitiesCellTable.this.dataProvider.getList().clear();
        PListInstanceActivitiesCellTable.this.dataProvider.getList().addAll(mInstanceActivityList);

        PListInstanceActivitiesCellTable.this.pProcessViewerPanel.setInstanceData(mInstanceActivityList);

        ConsoleEventBus.getEventBus().fireEvent(new RefreshDoneEvent());
      }

      public void onFailure(final Throwable caught) {
        if (PListInstanceActivitiesCellTable.this.tabLayoutPanel
                    .tabAlreadyExists(PListInstanceActivitiesCellTable.this.instanceId, PTab.Type.LIST_INSTANCE_ACTIVITY) != -1) {
          GWT.log("InstanceActivityGrid - refreshCurrentList: onFailure : " + caught.getMessage());
          ConsoleEventBus.getEventBus().fireEvent(new SuspendTimerEvent(true));
          if (caught instanceof OrchestraGwtException) {
            Popup.show((OrchestraGwtException) caught);
          } else {
            Popup.showError("Unable to access to Orchestra !", caught.getMessage());
          }
        }
      }
    });
  }

}
