package org.ow2.orchestra.console.GWTConsole.client.ProcessDetails;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.logical.shared.ValueChangeEvent;
import com.google.gwt.event.logical.shared.ValueChangeHandler;
import com.google.gwt.http.client.URL;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.RadioButton;
import java.util.List;
import org.ow2.orchestra.common.gwt.utils.client.OrchestraGwtException;
import org.ow2.orchestra.common.gwt.utils.client.ui.popup.Popup;
import org.ow2.orchestra.console.GWTConsole.client.ConsoleEventBus;
import org.ow2.orchestra.console.GWTConsole.client.Event.RefreshDoneEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.SuspendTimerEvent;
import org.ow2.orchestra.console.GWTConsole.client.ListInstance.MInstance;
import org.ow2.orchestra.console.GWTConsole.client.ListInstanceActivity.MInstanceActivity;
import org.ow2.orchestra.console.GWTConsole.client.ListProcessActivity.MProcessActivity;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServices;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServicesAsync;
import org.ow2.orchestra.console.GWTConsole.client.Tab.PTab;
import org.ow2.orchestra.console.GWTConsole.client.TabLayoutPanel.PTabLayoutPanel;

/**
 * Handles all processView related methods
 * (processView resizing and refresh).
 *
 * @author Guillaume Porcher
 */
public class PProcessViewerPanel {
  private final OrchestraManagementServicesAsync orchestramanagement = GWT.create(OrchestraManagementServices.class);

  private final VProcessDetailsIF view;
  private String processId;
  private final String instanceId;
  private final PTabLayoutPanel pTabLayoutPanel;

  private String instanceData;

  public PProcessViewerPanel(final String processId, final String instanceId, final VProcessDetailsIF view, final PTabLayoutPanel pTabLayoutPanel) {
    this.view = view;
    this.processId = processId;
    this.instanceId = instanceId;
    this.pTabLayoutPanel = pTabLayoutPanel;

    final FlowPanel fp = view.getButtonsPanel();
    final ValueChangeHandler<Boolean> vch = new ValueChangeHandler<Boolean>() {

      public void onValueChange(final ValueChangeEvent<Boolean> event) {
        if (event.getValue()) {
          final int index = Integer.parseInt(((RadioButton) event.getSource()).getFormValue());
          view.getDeckPanel().showWidget(index);
          ConsoleEventBus.getEventBus().fireEvent(new RefreshDoneEvent());
        }
      }
    };
    view.getDeckPanel().showWidget(1);
    final RadioButton rb1 = (RadioButton) fp.getWidget(0);
    rb1.addValueChangeHandler(vch);
    rb1.setVisible(false);
    final RadioButton rb2 = (RadioButton) fp.getWidget(1);
    rb2.addValueChangeHandler(vch);
    final RadioButton rb3 = (RadioButton) fp.getWidget(2);
    rb3.addValueChangeHandler(vch);

    if (instanceId != null) {
      this.orchestramanagement.getInstance(this.instanceId, new AsyncCallback<MInstance>() {
        public void onSuccess(final MInstance instance) {
          PProcessViewerPanel.this.processId = instance.getProcessId();
          PProcessViewerPanel.this.refreshProcessView();
        }

        public void onFailure(final Throwable caught) {
          GWT.log("PProcessViewerPanel - getProcessId : onFailure : " + caught.getMessage());
          if (caught instanceof OrchestraGwtException) {
            Popup.show((OrchestraGwtException) caught);
          } else {
            Popup.showError("Unable to access to Orchestra !", caught.getMessage());
          }
        }
      });
    } else {
      this.refreshProcessView();
    }
  }

  /**
   * Load console swf object,
   * fetch BPMN process model from orchestra and draw process.
   */
  public void refreshProcessView() {
    this.orchestramanagement.getProcessActivities(this.processId, new AsyncCallback<List<MProcessActivity>>() {
      public void onSuccess(final List<MProcessActivity> mProcessActivityList) {
        PProcessViewerPanel.this.setProcessData(mProcessActivityList);
      }

      public void onFailure(final Throwable caught) {
        GWT.log("PProcessViewerPanel - getProcessActivities: onFailure : " + caught.getMessage());
        if (caught instanceof OrchestraGwtException) {
          Popup.show((OrchestraGwtException) caught);
        } else {
          Popup.showError("Unable to access to Orchestra !", caught.getMessage());
        }
      }
    });


    this.orchestramanagement.getSerializedDesignerModel(this.processId, new AsyncCallback<String>() {

      public void onSuccess(final String result) {
        PProcessViewerPanel.this.setSerializedBPMNModel(result);
      }

      public void onFailure(final Throwable caught) {
        GWT.log("Process view - getDesignerModel: onFailure : " + caught.getMessage());
        if (PProcessViewerPanel.this.pTabLayoutPanel.tabAlreadyExists(PProcessViewerPanel.this.processId, PTab.Type.LIST_PROCESS_ACTIVITY) != -1) {
          ConsoleEventBus.getEventBus().fireEvent(new SuspendTimerEvent(true));
          if (caught instanceof OrchestraGwtException) {
            Popup.show((OrchestraGwtException) caught);
          } else {
            Popup.showError("Unable to access to Orchestra !", caught.getMessage());
          }
        }
      }
    });

  }

  private String getBPMNViewerId() {
    if (this.instanceId != null) {
      return "instance-view-" + this.instanceId;
    }
    return "process-view-" + this.processId;
  }

  public void setSerializedBPMNModel(final String serializedBPMNModel) {
    final HTML htmlPanel = PProcessViewerPanel.this.view.getBpmnContent();
    if (serializedBPMNModel != null && !"".equals(serializedBPMNModel)) {
      if (!PProcessViewerPanel.this.view.getButtonsPanel().getWidget(0).isVisible()) {
        // Show BPMN radio button and select it
        PProcessViewerPanel.this.view.getButtonsPanel().getWidget(0).setVisible(true);
        ((RadioButton) PProcessViewerPanel.this.view.getButtonsPanel().getWidget(0)).setValue(true, true);
      }
      if (htmlPanel.getHTML().isEmpty()) {
        // encode string in the html
        final String urlEncodedResult = URL.encodeQueryString(serializedBPMNModel);
        int height = Window.getClientHeight() - 200;
        htmlPanel.setHTML("<object classid=\"clsid:D27CDB6E-AE6D-11cf-96B8-444553540000\""
            + " id=\"" + PProcessViewerPanel.this.getBPMNViewerId() + "\"  width=\"100%\" height=\"" + height + "px\""
            + " codebase=\"http://fpdownload.macromedia.com/get/flashplayer/current/swflash.cab\">"
            + "<param name=\"movie\" value=\"console.swf\" />"
            + "<param name=\"FlashVars\" value=\"data=" + urlEncodedResult + ""
            + (instanceData != null ? ("&instanceData=" + instanceData) : "")
            + "\"/>"
            + " <embed src=\"console.swf\" FlashVars=\"data=" + urlEncodedResult + ""
            + (instanceData != null ? ("&instanceData=" + instanceData) : "")
            + "\""
            + " name=\"" + PProcessViewerPanel.this.getBPMNViewerId() + "\""
            + " type=\"application/x-shockwave-flash\" wmode=\"opaque\""
            + " pluginspage=\"http://www.adobe.com/go/getflashplayer\" width=\"100%\" height=\"" + height + "px\">"
            + "</object>");
      } else {
        try {
          this.setProcessDataInBPMNView(serializedBPMNModel, PProcessViewerPanel.this.getBPMNViewerId());
        } catch (final Exception caught) {
          GWT.log("Process view - getDesignerModel: onFailure : " + caught.getMessage());
          ConsoleEventBus.getEventBus().fireEvent(new SuspendTimerEvent(true));
          Popup.showError("Unable to access to Orchestra !", caught.getMessage());
        }
      }
    } else {
      PProcessViewerPanel.this.view.getButtonsPanel().getWidget(0).setVisible(false);
    }
    ConsoleEventBus.getEventBus().fireEvent(new RefreshDoneEvent());
  }

  private native void setProcessDataInBPMNView(String data, String viewId) /*-{
    var view;
    if (navigator.appName.indexOf("Microsoft") != -1) {
      view = $wnd[viewId];
    } else {
      view = $doc[viewId];
    }
    if (view != null && typeof(view.setData) == "function") {
      view.setData(data);
      if (this.@org.ow2.orchestra.console.GWTConsole.client.ProcessDetails.PProcessViewerPanel::instanceData != null) {
        view.setInstanceData(this.@org.ow2.orchestra.console.GWTConsole.client.ProcessDetails.PProcessViewerPanel::instanceData);
      }
    }
  }-*/;

  private native void setInstanceDataInBPMNView(String data, String viewId) /*-{
    var view;
    if (navigator.appName.indexOf("Microsoft") != -1) {
      view = $wnd[viewId];
    } else {
      view = $doc[viewId];
    }
    if (view != null && typeof(view.setInstanceData) == "function") {
      view.setInstanceData(data);
    } else {
      this.@org.ow2.orchestra.console.GWTConsole.client.ProcessDetails.PProcessViewerPanel::instanceData = data;
    }
  }-*/;

  public void setProcessData(final List<MProcessActivity> processActivities) {
    this.view.getProcessCanvas().setData(processActivities);
  }

  public void setInstanceData(final List<MInstanceActivity> instanceActivities) {
    this.view.getProcessCanvas().setInstanceData(instanceActivities);

    String data = "";
    for (final MInstanceActivity a : instanceActivities) {
      data += a.getInstanceActivityName() + "#" + a.getInstanceActivityState() + "##";
    }
    this.setInstanceDataInBPMNView(data, this.getBPMNViewerId());
  }
}
