/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.TabLayoutPanel;

import java.util.ArrayList;

import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.Style.Unit;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.logical.shared.CloseEvent;
import com.google.gwt.event.logical.shared.CloseHandler;
import com.google.gwt.event.logical.shared.ResizeEvent;
import com.google.gwt.event.logical.shared.ResizeHandler;
import com.google.gwt.event.logical.shared.SelectionEvent;
import com.google.gwt.event.logical.shared.SelectionHandler;
import com.google.gwt.user.client.Timer;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.RootLayoutPanel;
import com.google.gwt.user.client.ui.Widget;
import org.ow2.orchestra.console.GWTConsole.client.Commons.Commons;
import org.ow2.orchestra.console.GWTConsole.client.Commons.PresenterIF;
import org.ow2.orchestra.console.GWTConsole.client.ConsoleEventBus;
import org.ow2.orchestra.console.GWTConsole.client.DeployProcess.PDeployProcess;
import org.ow2.orchestra.console.GWTConsole.client.Event.RefreshDoneEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.RefreshDoneHandler;
import org.ow2.orchestra.console.GWTConsole.client.Event.SuspendTimerEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.SuspendTimerHandler;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshEvent;
import org.ow2.orchestra.console.GWTConsole.client.Finder.PFinder;
import org.ow2.orchestra.console.GWTConsole.client.InfoBar.PInfoBar;
import org.ow2.orchestra.console.GWTConsole.client.ListInstance.PListInstance;
import org.ow2.orchestra.console.GWTConsole.client.ListProcess.PListProcess;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServices;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServicesAsync;
import org.ow2.orchestra.console.GWTConsole.client.Tab.PTab;
import org.ow2.orchestra.console.GWTConsole.client.Tab.VTab;
import org.ow2.orchestra.console.GWTConsole.client.listJobs.PListJobs;
import org.ow2.orchestra.console.GWTConsole.client.listMessages.PListMessages;
import org.ow2.orchestra.console.GWTConsole.client.userTask.PListTasks;
import org.ow2.orchestra.console.GWTConsole.client.userTask.VListTasks;


public class PTabLayoutPanel {

  /**
   * @author Mathieu Crassin
   *
   * The interface that represent the view
   */
  private VTabLayoutPanelIF view = null;

  private final int refreshInterval = 30000;
  private Timer refreshTimer = null;
  protected boolean refreshTimerWorks;

  protected ArrayList<PresenterId> presenterList = null;

  protected OrchestraManagementServicesAsync orchestramanagement = GWT.create(OrchestraManagementServices.class);

  private class PresenterId {
    private final String elementId;
    private final PTab.Type elementType;
    private final PresenterIF presenter;

    public PresenterId(final String elementId, final PTab.Type elementType, final PresenterIF presenter) {
      this.elementId = elementId;
      this.elementType = elementType;
      this.presenter = presenter;
    }

    public String getElementId() {
      return this.elementId;
    }
    public PTab.Type getElementType() {
      return this.elementType;
    }
    public PresenterIF getPresenter() {
      return this.presenter;
    }
  }

  public PTabLayoutPanel() {
    //
    final VTabLayoutPanel vTabLayoutPanel = new VTabLayoutPanel();
    //
    this.bindingView(vTabLayoutPanel);
    //
    final PInfoBar pInfoBar = new PInfoBar(this.view.getVInfoBar());
    //
    this.createPresenters();
    //
    this.addTimer();
    this.addCloseWindowListener();
    this.addListener(pInfoBar);
    this.addSuspendTimerHandlerToBus();

    this.addHandlerToBus();
    this.adjustSize();

    final String res = Window.Location.getParameter("resize");
    if (res == null || !res.equals("false")) {
      Window.addResizeHandler(new ResizeHandler() {

        public void onResize(final ResizeEvent arg0) {
          RootLayoutPanel.get().setHeight("100%");
          RootLayoutPanel.get().setWidth("100%");
          PTabLayoutPanel.this.adjustSize();
        }
      });
    }
  }


  /**
   *
   */
  private void addSuspendTimerHandlerToBus() {
    ConsoleEventBus.getEventBus().addHandler(SuspendTimerEvent.TYPE, new SuspendTimerHandler() {
      public void onSuspend(final SuspendTimerEvent suspendTimerEvent) {
        if (suspendTimerEvent.getSuspend()) {
          PTabLayoutPanel.this.refreshTimer.cancel();
          PTabLayoutPanel.this.refreshTimerWorks  = false;
        } else {
          final String refresh = Window.Location.getParameter("refresh");
          if (refresh != null && refresh.equals("true")) {
            PTabLayoutPanel.this.refreshTimer.scheduleRepeating(PTabLayoutPanel.this.refreshInterval);
            PTabLayoutPanel.this.refreshTimerWorks = true;
          }
        }
      }
    });
  }


  /**
   *
   */
  private void addCloseWindowListener() {
    if (PTabLayoutPanel.getUserAgent().contains("msie")) {
      Window.addCloseHandler(new CloseHandler<Window>() {
        public void onClose(final CloseEvent arg0) {
          PTabLayoutPanel.this.refreshTimer.cancel();
        }
      });
    }
  }


  /**
   * Add the PTabLayoutPanel timer that refresh the active tab
   *
   * @author Mathieu Crassin
   */
  private void addTimer() {

    this.refreshTimer = new Timer() {
      @Override
      public void run() {
        int selectedIndex = PTabLayoutPanel.this.view.getTabLayoutPanel().getSelectedIndex();
        VTab selectedTab = (VTab) PTabLayoutPanel.this.view.getTabLayoutPanel().getTabWidget(selectedIndex);
        ConsoleEventBus.getEventBus().fireEvent(
            new ViewRefreshEvent(selectedTab.getElementId(), selectedTab.getElementType(), true));
      }
    };

    final String res = Window.Location.getParameter("refresh");

    if (res != null && res.equals("true")
        || res == null) {

      System.out.println("refresh active");

      this.refreshTimer.scheduleRepeating(this.refreshInterval);
      this.refreshTimerWorks = true;
    } else {
      System.out.println("refresh inactive");
    }
  }


  /**
   * Add selection tab listener
   *
   * @author Mathieu Crassin
   */
  private void addListener(final PInfoBar pInfoBar) {
    this.view.getTabLayoutPanel().addSelectionHandler(new SelectionHandler<Integer>() {
      public void onSelection(final SelectionEvent<Integer> arg0) {
        final String refresh = Window.Location.getParameter("refresh");
        if (PTabLayoutPanel.this.refreshTimerWorks || (refresh != null && refresh.equals("false"))) {
          int selectedIndex = PTabLayoutPanel.this.view.getTabLayoutPanel().getSelectedIndex();
          VTab selectedTab = (VTab) PTabLayoutPanel.this.view.getTabLayoutPanel().getTabWidget(selectedIndex);
          ConsoleEventBus.getEventBus().fireEvent(
              new ViewRefreshEvent(selectedTab.getElementId(), selectedTab.getElementType(), false));
        }
      }
    });
    //
    pInfoBar.getView().getRefreshLabel().addClickHandler(new ClickHandler() {
      public void onClick(final ClickEvent arg0) {
        ConsoleEventBus.getEventBus().fireEvent(new SuspendTimerEvent(false));
        int selectedIndex = PTabLayoutPanel.this.view.getTabLayoutPanel().getSelectedIndex();
        VTab selectedTab = (VTab) PTabLayoutPanel.this.view.getTabLayoutPanel().getTabWidget(selectedIndex);
        ConsoleEventBus.getEventBus().fireEvent(
                new ViewRefreshEvent(selectedTab.getElementId(), selectedTab.getElementType(), true));
      }
    });
  }

  /**
   * @author Mathieu Crassin
   * @param dp interface that represent the view of our application
   *
   * Attach our interface of the view with the real class
   */
  private void bindingView(final VTabLayoutPanelIF dp) {
    this.view = dp;
  }

  /**
   * Create the presenter of the view containing in this panel.
   *
   * @author Mathieu Crassin
   */
  private void createPresenters() {
    final PListProcess plp = new PListProcess(this.view.getVListProcess(), this);
    new PListInstance(this.view.getVListInstance(), this);
    new PDeployProcess(this.view.getVDeployProcess(),plp);
    new PFinder(this, this.view.getVFinder());
    new PListJobs(this.view.getVListJobs());
    new PListMessages(this.view.getVListMessages());
    this.orchestramanagement.isTaskSupported(new AsyncCallback<Boolean>() {
      public void onFailure(Throwable throwable) {
      }
      public void onSuccess(Boolean isSupported) {
        if (isSupported) {
          VListTasks tasksView = new VListTasks();
          VTab vTab = new VTab(Commons.TAB_TASKS, "Tasks", PTab.Type.LIST_TASKS);
          vTab.setDisplayCloseButton(false);
          vTab.setTabLabel("Tasks");

          // insert tab in the tab layout panel
          PTabLayoutPanel.this.view.getTabLayoutPanel().add(tasksView, vTab);
          new PListTasks(tasksView);
        }
      }
    });
    this.addTabID();
  }



  /**
   * Define Id for the first tabs
   *
   * @author Jean-Yves Goubet
   */
  private void addTabID() {
    this.view.getTabLayoutPanel().getTabWidget(0).ensureDebugId(Commons.TAB_DASHBOARD);
    this.view.getTabLayoutPanel().getTabWidget(1).ensureDebugId(Commons.TAB_PROCESS);
    this.view.getTabLayoutPanel().getTabWidget(2).ensureDebugId(Commons.TAB_INSTANCE);
    this.view.getTabLayoutPanel().getTabWidget(3).ensureDebugId(Commons.TAB_FINDER);
    this.view.getTabLayoutPanel().getTabWidget(4).ensureDebugId(Commons.TAB_JOB);
    this.view.getTabLayoutPanel().getTabWidget(5).ensureDebugId(Commons.TAB_MESSAGE);
  }


  /**
   * Function that manage the add tab task.
   * First the function check if the tab already exists and
   * then create the presenter & the view of the tab.
   *
   * @author Mathieu Crassin
   */
  public void addTab(final String elementId, final String elementName, final PTab.Type type, final Widget widget, final PresenterIF presenter) {
    // check tab already exist
    final int res = this.tabAlreadyExists(elementId, type);

    // tab doesn't exists
    if (res == -1) {
      // add the new tab
      final PTab pres = new PTab(elementName, elementId, type, widget, presenter, this.view.getTabLayoutPanel());

      // keep a pointer to the presenter
      this.addPresenter(elementId, type, pres);

      // select it
      this.view.getTabLayoutPanel().selectTab(this.view.getTabLayoutPanel().getWidgetCount() - 1);
    }
    // else
    else {
      // move it to the first pos.
      if (res >= 2) {
        // select it
        this.view.getTabLayoutPanel().selectTab(res);
      }
    }
  }


  /**
   * Function that search in the existing process tab list a process.
   *
   * @return the index of the tab element if found, else -1
   */
  public int tabAlreadyExists(final String processID, final PTab.Type type) {
    // this method is probably slow, and could be replace by a search in a list
    for(int i = 2; i < this.view.getTabLayoutPanel().getWidgetCount(); i++) {
      // cast the result of the getTabWidget (the header)
      final VTab tmp = (VTab)this.view.getTabLayoutPanel().getTabWidget(i);
      // compare with the research param.
      if (processID.equals(tmp.getElementId()) && type == tmp.getElementType()) {
        return i;
      }
    }
    return -1;
  }


  /**
   * This method is called to close tab if its corresponding process is undeployed.
   *
   * @param elementId : UUID of the element to remove.
   * @param type : type of the element to remove.
   *
   * @author Jean-Yves Goubet
   */
  public void removeTab(final String elementId, final PTab.Type type) {
    final int indexTab = this.tabAlreadyExists(elementId, type);
    if (indexTab != -1) {
      // get the presenter that match with the view
      final PresenterIF pres = this.findPresenter(elementId, type);
      // call the destroy method
      pres.destroy();
      // remove presenter from the list
      this.removePresenter(elementId, type);

      this.view.getTabLayoutPanel().remove(indexTab);
    }
  }

  /**
   * Get a string that represente the browser name.
   *
   * @author Mathieu Crassin
   * @return
   *
   */
  private static native String getUserAgent() /*-{
    return navigator.userAgent.toLowerCase();
  }-*/;


  /**
   *
   * @return
   */
  public VTabLayoutPanelIF getView() {
    return this.view;
  }


  /**
   * Method to get the height of content displayed
   *
   * @return
   */
  public int getHeightContent() {
    return this.view.getTabLayoutPanel().getWidget(this.view.getTabLayoutPanel().getSelectedIndex()).getOffsetHeight() + 174;
  }

  public int getWidthContent() {
    return this.view.getTabLayoutPanel().getWidget(this.view.getTabLayoutPanel().getSelectedIndex()).getOffsetWidth();
  }


  public void addPresenter(final String elementId, final PTab.Type elementType, final PresenterIF presenter) {
 // System.out.println("ADD PRESENTER TO LIST");

     final PresenterId presId = new PresenterId(elementId, elementType, presenter);
     if (this.presenterList == null) {
       this.presenterList = new ArrayList<PresenterId>();
     }

     if (this.recherchePresenter(elementId, elementType) == -1) {
       this.presenterList.add(presId);
     }

     System.out.println("Nb = " + this.presenterList.size());
   }

   public PresenterIF findPresenter(final String elementId, final PTab.Type elementType) {
 // System.out.println("FIND PRESENTER id = " + elementId + " & name = " + elementType);

     for (int i = 0; i < this.presenterList.size(); i++) {
       final PresenterId tmp = this.presenterList.get(i);

       if (tmp.getElementId().equals(elementId)
           && tmp.getElementType().equals(elementType)) {

         return tmp.getPresenter();
       }
     }

     return null;
   }
   public void removePresenter(final String elementId, final PTab.Type elementType) {
     final int i = this.recherchePresenter(elementId, elementType);
     this.presenterList.remove(i);
   }
   private int recherchePresenter(final String elementId, final PTab.Type elementType) {
     int i = 0;
     boolean found = false;
     while (i < this.presenterList.size() && !found) {
       final PresenterId tmp = this.presenterList.get(i);

       if (tmp.getElementId().equals(elementId)
           && tmp.getElementType().equals(elementType)) {

         found = true;
       } else {
         i++;
       }
     }

     if (found) {
       return i;
     } else {
       return -1;
     }
   }

  /**
   * This method is used to adjust the width and height of appli depending on
   * the client's size window and the tab layout content size too.
   *
   * @author Jean-Yves Goubet
   */
  private void adjustSize() {
    // check if the resize parameter is set or not
    final String res = Window.Location.getParameter("resize");
    // first of all, get the window widthClient
    final int widthClient = Window.getClientWidth();
    int widthContent = this.getWidthContent();
    // and window height
    final int heightClient = Window.getClientHeight();
    int heightContent = this.getHeightContent();

    GWT.log("resizing: " + widthClient + "/" + widthContent + " " + heightClient + "/" + heightContent);
    if (res != null && res.equals("false")) {
      // just for tests
      if (widthClient > 924) {
        // in this case, we just add some margin
        this.view.getLayoutPanel().setWidgetLeftWidth(this.view.getTabLayoutPanel(),
                (widthClient - 924) / 2, Unit.PX, 924, Unit.PX);
      } else {
        // and in this case not
        this.view.getLayoutPanel().setWidgetLeftWidth(this.view.getTabLayoutPanel(),
                0, Unit.PX, 924, Unit.PX);
      }
      this.view.getLayoutPanel().setWidth("924px");
    } else {
      RootLayoutPanel.get().setWidth(Math.max(widthContent, widthClient) + "px");
    }
    if (res != null && res.equals("false")) {
      // just for tests
      this.view.getLayoutPanel().setHeight("600px");
    } else {
      // otherwise
      RootLayoutPanel.get().setHeight(Math.max(heightContent, heightClient) + "px");
    }
  }

  private void addHandlerToBus() {
    ConsoleEventBus.getEventBus().addHandler(RefreshDoneEvent.TYPE, new RefreshDoneHandler() {
      public void onRefreshDone(final RefreshDoneEvent refreshDoneEvent) {
        adjustSize();
        if (PTabLayoutPanel.this.refreshTimerWorks)  {
          PTabLayoutPanel.this.refreshTimer.scheduleRepeating(PTabLayoutPanel.this.refreshInterval);
        }
      }
    });
  }
}
