/**
 * Copyright (C) 2011  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.console.GWTConsole.client.listMessages;

import com.google.gwt.cell.client.Cell;
import com.google.gwt.cell.client.CompositeCell;
import com.google.gwt.cell.client.HasCell;
import com.google.gwt.core.client.GWT;
import com.google.gwt.safehtml.shared.SafeHtmlUtils;
import com.google.gwt.user.cellview.client.Column;
import com.google.gwt.user.cellview.client.ColumnSortEvent.ListHandler;
import com.google.gwt.user.cellview.client.IdentityColumn;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.view.client.ListDataProvider;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import org.ow2.orchestra.common.gwt.utils.client.OrchestraGwtException;
import org.ow2.orchestra.common.gwt.utils.client.ui.popup.Popup;
import org.ow2.orchestra.console.GWTConsole.client.Commons.celltable.ConsoleCellTable;
import org.ow2.orchestra.console.GWTConsole.client.Commons.celltable.HTMLCell;
import org.ow2.orchestra.console.GWTConsole.client.Commons.celltable.ImageActionCell;
import org.ow2.orchestra.console.GWTConsole.client.Commons.celltable.LabelCell;
import org.ow2.orchestra.console.GWTConsole.client.Commons.celltable.NullableDateCell;
import org.ow2.orchestra.console.GWTConsole.client.ConsoleEventBus;
import org.ow2.orchestra.console.GWTConsole.client.Event.RefreshDoneEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.SuspendTimerEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshHandler;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServices;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServicesAsync;
import org.ow2.orchestra.console.GWTConsole.client.Resources.ImageResources;
import org.ow2.orchestra.console.GWTConsole.client.Tab.PTab.Type;


/**
 * @author Guillaume Porcher
 *
 */
public class PListMessages {


  protected OrchestraManagementServicesAsync orchestramanagement = GWT.create(OrchestraManagementServices.class);

  // Create a data provider.
  private final ListDataProvider<MMessage> dataProvider = new ListDataProvider<MMessage>();

  private final String debugPrefix = "listMessages";

  public PListMessages(final VListMessages view) {
    //
    this.addViewRefreshHandlerToBus();

    view.getPager().setDisplay(view.getTable());
    view.getPager().ensureDebugId(debugPrefix + "Pager");

    view.getTable().ensureDebugId(debugPrefix);

    // Add a ColumnSortEvent.ListHandler to connect sorting to the
    // java.util.List.
    final ListHandler<MMessage> columnSortHandler = new ListHandler<MMessage>(this.dataProvider.getList());
    view.getTable().addColumnSortHandler(columnSortHandler);

    this.setColumns(view.getTable(), columnSortHandler);

    // Connect the table to the data provider.
    this.dataProvider.addDataDisplay(view.getTable());

    // Fetch data
    this.refresh();

  }

  private void setColumns(final ConsoleCellTable<MMessage> table, final ListHandler<MMessage> columnSortHandler) {
    /* ID column */
    final Column<MMessage, String> idColumn = new Column<MMessage, String>(new LabelCell(this.debugPrefix)) {
      @Override
      public String getValue(final MMessage mMessage) {
        return Long.toString(mMessage.getMessageId());
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, idColumn, "ID");

    /* ProcessQName column */
    final Column<MMessage, String> processQNameColumn = new Column<MMessage, String>(new LabelCell(this.debugPrefix)) {
      @Override
      public String getValue(final MMessage mMessage) {
        return mMessage.getProcessQName();
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, processQNameColumn, "Process QName");

    /* PortTypeQName column */
    final Column<MMessage, String> portTypeQNameColumn = new Column<MMessage, String>(new LabelCell(this.debugPrefix)) {
      @Override
      public String getValue(final MMessage mMessage) {
        return mMessage.getPortTypeQName();
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, portTypeQNameColumn, "Port type QName");

    /* Operation name column */
    final Column<MMessage, String> operationNameColumn = new Column<MMessage, String>(new LabelCell(this.debugPrefix)) {
      @Override
      public String getValue(final MMessage mMessage) {
        return mMessage.getOperationName();
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, operationNameColumn, "Operation Name");

    /* Is One way column */
    final Column<MMessage, String> isOneWayColumn = new Column<MMessage, String>(new LabelCell(this.debugPrefix)) {
      @Override
      public String getValue(final MMessage mMessage) {
        return Boolean.toString(mMessage.isOneWay());
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, isOneWayColumn, "One Way");

    /* Message column */
    final Column<MMessage, String> messageColumn = new Column<MMessage, String>(new HTMLCell(this.debugPrefix)) {
      @Override
      public String getValue(final MMessage mMessage) {
        return SafeHtmlUtils.htmlEscape(mMessage.getMessage()).replace("\n", "<br />");
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, messageColumn, "Content");
    /* Added date column */
    final Column<MMessage, Date> dueDateColumn = new Column<MMessage, Date>(new NullableDateCell(this.debugPrefix)) {
      @Override
      public Date getValue(final MMessage mMessage) {
        return mMessage.getAddedDate();
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, dueDateColumn, "Added Date");

    /* Actions column */
    final List<HasCell<MMessage, ?>> actionList = new ArrayList<HasCell<MMessage,?>>();
    actionList.add(new IdentityColumn<MMessage>(this.getRemoveActionCell()));

    final CompositeCell<MMessage> actionCells = new CompositeCell<MMessage>(actionList);
    final Column<MMessage, MMessage> actionsColumn = new IdentityColumn<MMessage>(actionCells);
    actionsColumn.setSortable(false);
    // Add the columns.
    table.addColumn(actionsColumn, "Actions");
  }

  private Cell<MMessage> getRemoveActionCell() {
    final Cell<MMessage> removeAction =
      new ImageActionCell<MMessage>(
              ImageResources.INSTANCE.deleteIcon(),
              ImageResources.INSTANCE.deleteIconDisabled(), "remove",
              this.debugPrefix + "-remove") {

        @Override
        public void execute(final MMessage object) {
          if (this.isActive(object)) {
            orchestramanagement.removePendingMessage(object, new AsyncCallback<Void>() {

              public void onSuccess(final Void arg0) {
                ConsoleEventBus.getEventBus().fireEvent(new ViewRefreshEvent(null,
                        Type.LIST_MESSAGES, true));
              }

              public void onFailure(final Throwable arg0) {
                if (arg0 instanceof OrchestraGwtException) {
                  Popup.show((OrchestraGwtException) arg0);
                } else {
                  Popup.showError("Fail while removing message !", arg0.getMessage());
                }
              }
            });
          }
        }

        @Override
        public boolean isActive(final MMessage value) {
          return true;
        }
      };
    return removeAction;
  }


  public void refresh() {
    this.orchestramanagement.getPendingMessages(new AsyncCallback<List<MMessage>>() {
      public void onSuccess(final List<MMessage> messages) {
        PListMessages.this.dataProvider.getList().clear();
        PListMessages.this.dataProvider.getList().addAll(messages);
        ConsoleEventBus.getEventBus().fireEvent(new RefreshDoneEvent());
      }

      public void onFailure(final Throwable caught) {
        GWT.log("MessagesCellTable - refreshCurrentList: onFailure : " + caught.getMessage());
        ConsoleEventBus.getEventBus().fireEvent(new SuspendTimerEvent(true));
        if (caught instanceof OrchestraGwtException) {
          Popup.show((OrchestraGwtException) caught);
        } else {
          Popup.showError("Unable to access to Orchestra !", caught.getMessage());
        }
      }
    });
  }

  /**
   * Wait refresh event on the eventBus to launch refreshCurrentList
   *
   * @author Jean-Yves Goubet
   */
  private void addViewRefreshHandlerToBus() {
    ConsoleEventBus.getEventBus().addHandler(ViewRefreshEvent.TYPE, new ViewRefreshHandler() {

      public void onRefresh(final ViewRefreshEvent viewRefreshEvent) {
        if (viewRefreshEvent.getElementType() == Type.LIST_MESSAGES) {
          PListMessages.this.refresh();
        }
      }
    });
  }
}
