/*
 * Copyright (C) 2012 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 */

package org.ow2.orchestra.console.GWTConsole.client.simulation;

import com.google.gwt.user.client.rpc.AsyncCallback;
import java.util.Collections;
import org.ow2.orchestra.common.gwt.utils.client.OrchestraGwtException;
import org.ow2.orchestra.common.gwt.utils.client.ui.popup.Popup;
import org.ow2.orchestra.console.GWTConsole.client.Commons.PresenterIF;
import org.ow2.orchestra.console.GWTConsole.client.Tab.PTab;
import org.ow2.orchestra.console.GWTConsole.client.TabLayoutPanel.PTabLayoutPanel;
import org.ow2.orchestra.console.ws.client.WSConsoleServices;
import org.ow2.orchestra.console.ws.client.model.ProcessEndpoints;

/**
 * @author Loic Albertin
 */
public class PProcessSimulation extends PresenterIF {
  private VProcessSimulationIF view;
  private String processId;

  public PProcessSimulation(final PTabLayoutPanel pTabLayoutPanel, final String processId, final String processName) {
    final VProcessSimulation view = new VProcessSimulation();
    this.bindingView(view);
    this.processId = processId;
    this.view.setPresenter(this);

    pTabLayoutPanel.addTab(this.processId, processName + " simulation", PTab.Type.SIMULATION, view, this);
    WSConsoleServices.App.getInstance().getProcessEndpoints(processId, new AsyncCallback<ProcessEndpoints>() {
      public void onFailure(Throwable caught) {
        if (caught instanceof OrchestraGwtException) {
          Popup.show((OrchestraGwtException) caught);
        } else {
          Popup.showError(caught.getMessage());
        }
      }

      public void onSuccess(ProcessEndpoints result) {
        PProcessSimulation.this.view.getProcessEndpointsExplorer().getPresenter().exploreProcessEndpoints(
            Collections.singletonList(result), false);
      }
    });
  }

  @Override
  protected void bindingView(Object viewToBind) {
    this.view = (VProcessSimulationIF) viewToBind;
  }

  @Override
  protected void addListeners() {
    //To change body of implemented methods use File | Settings | File Templates.
  }

  @Override
  protected void setDebugID() {
    //To change body of implemented methods use File | Settings | File Templates.
  }

  @Override
  public void destroy() {
    //To change body of implemented methods use File | Settings | File Templates.
  }
}
